@@ -13,7 +13,7 @@
  * See the main() function at the bottom that shows an app that retrives from a
  * specified url using fgets() and fread() and saves as two output files.
  *
- * Coyright (c)2003 Simtec Electronics
+ * Copyright (c) 2003 Simtec Electronics
  *
  * Re-implemented by Vincent Sanders <vince@kyllikki.org> with extensive
  * reference to original curl example code
@@ -45,7 +45,9 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <sys/time.h>
+#ifndef WIN32
+#  include <sys/time.h>
+#endif
 #include <stdlib.h>
 #include <errno.h>
 
@@ -129,7 +131,6 @@ fill_buffer(URL_FILE *file,int want,int waittime)
     fd_set fdread;
     fd_set fdwrite;
     fd_set fdexcep;
-    int maxfd;
     struct timeval timeout;
     int rc;
 
@@ -142,6 +143,7 @@ fill_buffer(URL_FILE *file,int want,int waittime)
     /* attempt to fill buffer */
     do
     {
+        int maxfd = -1;
         FD_ZERO(&fdread);
         FD_ZERO(&fdwrite);
         FD_ZERO(&fdexcep);
@@ -154,8 +156,10 @@ fill_buffer(URL_FILE *file,int want,int waittime)
         curl_multi_fdset(multi_handle, &fdread, &fdwrite, &fdexcep, &maxfd);
 
         /* In a real-world program you OF COURSE check the return code of the
-           function calls, *and* you make sure that maxfd is bigger than -1
-           so that the call to select() below makes sense! */
+           function calls.  On success, the value of maxfd is guaranteed to be
+           greater or equal than -1.  We call select(maxfd + 1, ...), specially
+           in case of (maxfd == -1), we call select(0, ...), which is basically
+           equal to sleep. */
 
         rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &timeout);
 
@@ -219,7 +223,7 @@ url_fopen(const char *url,const char *operation)
     URL_FILE *file;
     (void)operation;
 
-    file = (URL_FILE *)malloc(sizeof(URL_FILE));
+    file = malloc(sizeof(URL_FILE));
     if(!file)
         return NULL;
 
