@@ -5,7 +5,6 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: multi-post.c,v 1.5 2007-07-12 21:11:10 danf Exp $
  *
  * This is an example application source code using the multi interface
  * to do a multipart formpost without "blocking".
@@ -28,7 +27,7 @@ int main(int argc, char *argv[])
   struct curl_slist *headerlist=NULL;
   static const char buf[] = "Expect:";
 
-  /* Fill in the file upload field. This makes libcurl load data from  
+  /* Fill in the file upload field. This makes libcurl load data from
      the given file name when curl_easy_perform() is called. */
   curl_formadd(&formpost,
                &lastptr,
@@ -61,7 +60,7 @@ int main(int argc, char *argv[])
     /* what URL that receives this POST */
     curl_easy_setopt(curl, CURLOPT_URL,
                      "http://www.fillinyoururl.com/upload.cgi");
-    curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
+    curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
     curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headerlist);
     curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
@@ -78,7 +77,7 @@ int main(int argc, char *argv[])
       fd_set fdread;
       fd_set fdwrite;
       fd_set fdexcep;
-      int maxfd;
+      int maxfd = -1;
 
       FD_ZERO(&fdread);
       FD_ZERO(&fdwrite);
@@ -92,8 +91,10 @@ int main(int argc, char *argv[])
       curl_multi_fdset(multi_handle, &fdread, &fdwrite, &fdexcep, &maxfd);
 
       /* In a real-world program you OF COURSE check the return code of the
-         function calls, *and* you make sure that maxfd is bigger than -1
-         so that the call to select() below makes sense! */
+         function calls.  On success, the value of maxfd is guaranteed to be
+         greater or equal than -1.  We call select(maxfd + 1, ...), specially in
+         case of (maxfd == -1), we call select(0, ...), which is basically equal
+         to sleep. */
 
       rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &timeout);
 
