@@ -6,7 +6,7 @@
 #                            | (__| |_| |  _ <| |___
 #                             \___|\___/|_| \_\_____|
 #
-# Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
+# Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
@@ -19,14 +19,14 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: testcurl.pl,v 1.74 2009-05-12 11:24:29 yangtse Exp $
 ###########################################################################
 
 ###########################
 #  What is This Script?
 ###########################
 
-# testcurl.pl is the master script to use for automatic testing of CVS-curl.
+# testcurl.pl is the master script to use for automatic testing of curl
+# directly off its source repository.
 # This is written for the purpose of being run from a crontab job or similar
 # at a regular interval. The output is suitable to be mailed to
 # curl-autocompile@haxx.se to be dealt with automatically (make sure the
@@ -46,13 +46,14 @@
 # --extvercmd=[command]    Command to use for displaying version with cross compiles.
 # --mktarball=[command]    Command to run after completed test
 # --name=[name]            Set name to report as
-# --nocvsup                Don't update from CVS even though it is a CVS tree
+# --nocvsup                Don't pull from git even though it is a git tree
+# --nogitpull              Don't pull from git even though it is a git tree
 # --nobuildconf            Don't run buildconf
 # --runtestopts=[options]  Options to pass to runtests.pl
 # --setup=[file name]      File name to read setup from (deprecated)
 # --target=[your os]       Specify your target environment.
 #
-# if [curl-daily-name] is omitted, a 'curl' CVS directory is assumed.
+# if [curl-daily-name] is omitted, a 'curl' git directory is assumed.
 #
 
 use strict;
@@ -62,20 +63,26 @@ use Cwd;
 # Turn on warnings (equivalent to -w, which can't be used with /usr/bin/env)
 #BEGIN { $^W = 1; }
 
-use vars qw($version $fixed $infixed $CURLDIR $CVS $pwd $build $buildlog
+use vars qw($version $fixed $infixed $CURLDIR $git $pwd $build $buildlog
             $buildlogname $configurebuild $targetos $confsuffix $binext
             $libext);
+
 use vars qw($name $email $desc $confopts $runtestopts $setupfile $mktarball
-            $extvercmd $nocvsup $nobuildconf $crosscompile $timestamp);
+            $extvercmd $nogitpull $nobuildconf $crosscompile
+            $timestamp);
 
 # version of this script
-$version='$Revision: 1.74 $';
+$version='2010-04-13';
 $fixed=0;
 
-# Determine if we're running from CVS or a canned copy of curl,
+# Determine if we're running from git or a canned copy of curl,
 # or if we got a specific target option or setup file option.
 $CURLDIR="curl";
-$CVS=1;
+if (-f ".git/config") {
+  $CURLDIR = "./";
+}
+
+$git=1;
 $setupfile = 'setup';
 while ($ARGV[0]) {
   if ($ARGV[0] =~ /--target=/) {
@@ -102,8 +109,8 @@ while ($ARGV[0]) {
   elsif ($ARGV[0] =~ /--configure=/) {
     $confopts = (split(/=/, shift @ARGV))[1];
   }
-  elsif ($ARGV[0] =~ /--nocvsup/) {
-    $nocvsup=1;
+  elsif (($ARGV[0] eq "--nocvsup") || ($ARGV[0] eq "--nogitpull")) {
+    $nogitpull=1;
     shift @ARGV;
   }
   elsif ($ARGV[0] =~ /--nobuildconf/) {
@@ -119,7 +126,7 @@ while ($ARGV[0]) {
   }
   else {
     $CURLDIR=shift @ARGV;
-    $CVS=0;
+    $git=0; # a given dir, assume not using git
   }
 }
 
@@ -137,7 +144,7 @@ if ($^O eq 'MSWin32' || $targetos) {
     $binext = '.exe';
     $libext = '.lib';
   }
-  elsif ($targetos =~ /mingw32/) {
+  elsif ($targetos =~ /mingw/) {
     $binext = '.exe';
     if ($^O eq 'MSWin32') {
       $libext = '.a';
@@ -228,6 +235,22 @@ sub mydie($){
     exit 1;
 }
 
+sub get_host_triplet {
+  my $triplet;
+  my $configfile = "$pwd/$build/lib/curl_config.h";
+
+  if(-f $configfile && -s $configfile && open(LIBCONFIGH, "<$configfile")) {
+    while(<LIBCONFIGH>) {
+      if($_ =~ /^\#define\s+OS\s+"*([^"][^"]*)"*\s*/) {
+        $triplet = $1;
+        last;
+      }
+    }
+    close(LIBCONFIGH);
+  }
+  return $triplet;
+}
+
 if (open(F, "$setupfile")) {
   while (<F>) {
     if (/(\w+)=(.*)/) {
@@ -306,6 +329,7 @@ logit "CFLAGS = ".$ENV{CFLAGS};
 logit "LDFLAGS = ".$ENV{LDFLAGS};
 logit "CC = ".$ENV{CC};
 logit "MAKEFLAGS = ".$ENV{MAKEFLAGS};
+logit "PKG_CONFIG_PATH = ".$ENV{PKG_CONFIG_PATH};
 logit "target = ".$targetos;
 logit "version = $version"; # script version
 logit "date = $timestamp";  # When the test build starts
@@ -317,15 +341,15 @@ $str1066os = undef;
 $pwd = getcwd();
 
 if (-d $CURLDIR) {
-  if ($CVS && -d "$CURLDIR/CVS") {
-    logit "$CURLDIR is verified to be a fine source dir";
+  if ($git && -d "$CURLDIR/.git") {
+    logit "$CURLDIR is verified to be a fine git source dir";
     # remove the generated sources to force them to be re-generated each
     # time we run this test
     unlink "$CURLDIR/src/hugehelp.c";
-  } elsif (!$CVS && -f "$CURLDIR/tests/testcurl.pl") {
+  } elsif (!$git && -f "$CURLDIR/tests/testcurl.pl") {
     logit "$CURLDIR is verified to be a fine daily source dir"
   } else {
-    mydie "$CURLDIR is not a daily source dir or checked out from CVS!"
+    mydie "$CURLDIR is not a daily source dir or checked out from git!"
   }
 }
 $build="build-$$";
@@ -351,55 +375,46 @@ if (-d $build) {
 # get in the curl source tree root
 chdir $CURLDIR;
 
-# Do the CVS thing, or not...
-if ($CVS) {
-
-  # this is a temporary fix to make things work again, remove later
-  logit "remove ares/aclocal.m4";
-  unlink "ares/aclocal.m4";
-
-  logit "update from CVS";
-  my $cvsstat;
-
-  sub cvsup() {
-    # update quietly to the latest CVS
-    if($nocvsup) {
-        logit "Skipping CVS update (--nocvsup)";
-        return 1;
+# Do the git thing, or not...
+if ($git) {
+  # update quietly to the latest git
+  if($nogitpull) {
+    logit "skipping git pull (--nogitpull)";
+  } else {
+    my $gitstat = 0;
+    my @commits;
+    logit "run git pull in curl";
+    system("git pull 2>&1");
+    $gitstat += $?;
+    logit "failed to update from curl git ($?), continue anyway" if ($?);
+    # get the last 5 commits for show (even if no pull was made)
+    @commits=`git log --pretty=oneline --abbrev-commit -5`;
+    logit "The most recent curl git commits:";
+    for (@commits) {
+      chomp ($_);
+      logit "  $_";
     }
-    else {
-        logit "run cvs up";
-        system("cvs -Q up -dP 2>&1");
+    if (-d "ares/.git") {
+      chdir "ares";
+      logit "run git pull in ares";
+      system("git pull 2>&1");
+      $gitstat += $?;
+      logit "failed to update from ares git ($?), continue anyway" if ($?);
+      # get the last 5 commits for show (even if no pull was made)
+      @commits=`git log --pretty=oneline --abbrev-commit -5`;
+      logit "The most recent ares git commits:";
+      for (@commits) {
+        chomp ($_);
+        logit "  $_";
+      }
+      chdir "$pwd/$CURLDIR";
     }
-
-    $cvsstat=$?;
-
-    # return !RETURNVALUE so that errors return 0 while goodness
-    # returns 1
-    return !$cvsstat;
-  }
-
-  my $att=0;
-  while (!cvsup()) {
-    $att++;
-    logit "failed CVS update attempt number $att.";
-    if ($att > 20) {
-      $cvsstat=111;
-      last; # get out of the loop
-    }
-    sleep 5;
-  }
-
-  if ($cvsstat != 0) {
-    mydie "failed to update from CVS ($cvsstat), exiting";
-  }
-  elsif (!$nocvsup) {
-    # Set timestamp to the UTC the CVS update took place.
-    $timestamp = scalar(gmtime)." UTC";
+    # Set timestamp to the UTC the git update took place.
+    $timestamp = scalar(gmtime)." UTC" if (!$gitstat);
   }
 
   if($nobuildconf) {
-      logit "told to not run buildconf";
+    logit "told to not run buildconf";
   }
   elsif ($configurebuild) {
     # remove possible left-overs from the past
@@ -430,14 +445,14 @@ if ($CVS) {
   }
 }
 
-# Set timestamp to the one in curlver.h if this isn't a CVS test build.
+# Set timestamp to the one in curlver.h if this isn't a git test build.
 if ((-f "include/curl/curlver.h") &&
     (open(F, "<include/curl/curlver.h"))) {
   while (<F>) {
     chomp;
     if ($_ =~ /^\#define\s+LIBCURL_TIMESTAMP\s+\"(.+)\".*$/) {
       my $stampstring = $1;
-      if ($stampstring !~ /CVS/) {
+      if ($stampstring !~ /DEV/) {
           $stampstring =~ s/\s+UTC//;
           $timestamp = $stampstring." UTC";
       }
@@ -463,10 +478,8 @@ if ($configurebuild) {
 sub findinpath {
   my $c;
   my $e;
-  my $x='';
-  $x='.exe' if ($^O eq 'MSWin32');
-  my $s=':';
-  $s=';' if ($^O eq 'MSWin32');
+  my $x = ($^O eq 'MSWin32') ? '.exe' : '';
+  my $s = ($^O eq 'MSWin32') ? ';' : ':';
   my $p=$ENV{'PATH'};
   my @pa = split($s, $p);
   for $c (@_) {
@@ -482,6 +495,8 @@ my $make = findinpath("gmake", "make", "nmake");
 if(!$make) {
     mydie "Couldn't find make in the PATH";
 }
+# force to 'nmake' for VC builds
+$make = "nmake" if ($targetos =~ /vc/);
 logit "going with $make as make";
 
 # change to build dir
@@ -549,14 +564,14 @@ else {
   mydie "no curlbuild.h created/found";
 }
 
-logit_spaced "display lib/config$confsuffix.h";
-open(F, "lib/config$confsuffix.h") or die "lib/config$confsuffix.h: $!";
+logit_spaced "display lib/curl_config$confsuffix.h";
+open(F, "lib/curl_config$confsuffix.h") or die "lib/curl_config$confsuffix.h: $!";
 while (<F>) {
   print if /^ *#/;
 }
 close(F);
 
-if (grepfile("define USE_ARES", "lib/config$confsuffix.h")) {
+if (grepfile("define USE_ARES", "lib/curl_config$confsuffix.h")) {
   print "\n";
   logit "setup to build ares";
 
@@ -585,8 +600,8 @@ if (grepfile("define USE_ARES", "lib/config$confsuffix.h")) {
     mydie "no ares_build.h created/found";
   }
 
-  logit_spaced "display ares/config$confsuffix.h";
-  if(open(F, "ares/config$confsuffix.h")) {
+  logit_spaced "display ares/ares_config$confsuffix.h";
+  if(open(F, "ares/ares_config$confsuffix.h")) {
       while (<F>) {
           print if /^ *#/;
       }
@@ -614,41 +629,21 @@ if (grepfile("define USE_ARES", "lib/config$confsuffix.h")) {
   if (-f "libcares$libext") {
     logit "ares is now built successfully (libcares$libext)";
   } else {
-    logit "ares build failed (libares$libext)";
+    mydie "ares build failed (libcares$libext)";
   }
 
   # cd back to the curl build dir
   chdir "$pwd/$build";
 }
 
-if ($configurebuild) {
-  logit "$make -i";
-  open(F, "$make -i 2>&1 |") or die;
-  while (<F>) {
-    s/$pwd//g;
-    print;
-  }
-  close(F);
-}
-else {
-  logit "$make -i $targetos";
-  if ($^O eq 'MSWin32') {
-    if ($targetos =~ /vc/) {
-      open(F, "nmake -i $targetos|") or die;
-    }
-    else {
-      open(F, "$make -i $targetos |") or die;
-    }
-  }
-  else {
-    open(F, "$make -i $targetos 2>&1 |") or die;
-  }
-  while (<F>) {
-    s/$pwd//g;
-    print;
-  }
-  close(F);
+my $mkcmd = "$make -i" . ($targetos && !$configurebuild ? " $targetos" : "");
+logit "$mkcmd";
+open(F, "$mkcmd 2>&1 |") or die;
+while (<F>) {
+  s/$pwd//g;
+  print;
 }
+close(F);
 
 if (-f "lib/libcurl$libext") {
   logit "libcurl was created fine (libcurl$libext)";
@@ -669,8 +664,9 @@ if (!$crosscompile || (($extvercmd ne '') && (-x $extvercmd))) {
   my $cmd = ($extvercmd ne '' ? $extvercmd.' ' : '')."./src/curl${binext} --version|";
   open(F, $cmd);
   while(<F>) {
-      print;
-      print LOG;
+    # strip CR from output on non-win32 platforms (wine on Linux)
+    s/\r// if ($^O ne 'MSWin32');
+    print;
   }
   close(F);
 }
@@ -702,11 +698,28 @@ if ($configurebuild && !$crosscompile) {
   } else {
     logit "the tests were successful!";
   }
-} else {
-  # dummy message to feign success
+}
+else {
   if($crosscompile) {
-    logit "cross-compiling, can't run tests";
+    # build test harness programs for selected cross-compiles
+    my $host_triplet = get_host_triplet();
+    if($host_triplet =~ /([^-]+)-([^-]+)-mingw(.*)/) {
+      chdir "$pwd/$build/tests";
+      logit_spaced "build test harness";
+      open(F, "$make -i 2>&1 |") or die;
+      open(LOG, ">$buildlog") or die;
+      while (<F>) {
+        s/$pwd//g;
+        print;
+        print LOG;
+      }
+      close(F);
+      close(LOG);
+      chdir "$pwd/$build";
+    }
+    logit_spaced "cross-compiling, can't run tests";
   }
+  # dummy message to feign success
   print "TESTDONE: 1 tests out of 0 (dummy message)\n";
 }
 
